@@ -1,6 +1,6 @@
 ;;; Continuation-passing style (CPS) intermediate language (IL)
 
-;; Copyright (C) 2013, 2014, 2015, 2017, 2018 Free Software Foundation, Inc.
+;; Copyright (C) 2013-2021 Free Software Foundation, Inc.
 
 ;;;; This library is free software; you can redistribute it and/or
 ;;;; modify it under the terms of the GNU Lesser General Public
@@ -40,34 +40,42 @@
   #:use-module (language cps with-cps)
   #:export (contify))
 
-(define (compute-singly-referenced-labels conts)
-  "Compute the set of labels in CONTS that have exactly one
-predecessor."
-  (define (add-ref label cont single multiple)
-    (define (ref k single multiple)
-      (if (intset-ref single k)
-          (values single (intset-add! multiple k))
-          (values (intset-add! single k) multiple)))
-    (define (ref0) (values single multiple))
-    (define (ref1 k) (ref k single multiple))
-    (define (ref2 k k*)
-      (if k*
-          (let-values (((single multiple) (ref k single multiple)))
-            (ref k* single multiple))
-          (ref1 k)))
-    (match cont
-      (($ $kreceive arity k) (ref1 k))
-      (($ $kfun src meta self ktail kclause) (ref2 ktail kclause))
-      (($ $ktail) (ref0))
-      (($ $kclause arity kbody kalt) (ref2 kbody kalt))
-      (($ $kargs names syms ($ $continue k)) (ref1 k))
-      (($ $kargs names syms ($ $branch kf kt)) (ref2 kf kt))
-      (($ $kargs names syms ($ $prompt k kh)) (ref2 k kh))
-      (($ $kargs names syms ($ $throw)) (ref0))))
-  (let*-values (((single multiple) (values empty-intset empty-intset))
-                ((single multiple) (intmap-fold add-ref conts single multiple)))
-    (intset-subtract (persistent-intset single)
-                     (persistent-intset multiple))))
+(define (compute-first-class-functions conts)
+  "Compute the set of $kfun labels in @var{conts} that can be called by
+value rather than by label.  Assumes @var{conts} contains only reachable
+conts.  Assumes each $kfun is only made into a first class value by a
+single label.  Returns an intmap map from $kfun label to label in which
+the first-class function is defined."
+  (define (add kdef kfun first-class)
+    (intmap-add! first-class kfun kdef))
+  (persistent-intmap
+   (intmap-fold
+    (lambda (label cont first-class)
+      (match cont
+        (($ $kargs _ _ ($ $continue k src ($ $fun kfun)))
+         (add label kfun first-class))
+        (($ $kargs _ _ ($ $continue k src
+                          ($ $rec _ vars (($ $fun kfuns) ...))))
+         (fold (lambda (kfun first-class)
+                 (add label kfun first-class))
+               first-class
+               kfuns))
+        (_ first-class)))
+    conts
+    empty-intmap)))
+
+(define (compute-functions-called-by-label conts)
+  "Compute the set of $kfun labels in @var{conts} which are targets of
+$callk."
+  (persistent-intset
+   (intmap-fold
+    (lambda (label cont by-label)
+      (match cont
+        (($ $kargs _ _ ($ $continue k src ($ $callk kfun)))
+         (intset-add! by-label kfun))
+        (_ by-label)))
+    conts
+    empty-intset)))
 
 (define (compute-functions conts)
   "Compute a map from $kfun label to bound variable names for all
@@ -79,27 +87,57 @@ the set."
   (define (function-self label)
     (match (intmap-ref conts label)
       (($ $kfun src meta self) self)))
-  (let ((single (compute-singly-referenced-labels conts)))
-    (intmap-fold (lambda (label cont functions)
-                   (match cont
-                     (($ $kargs _ _ ($ $continue k src ($ $fun kfun)))
-                      (if (intset-ref single k)
-                          (match (intmap-ref conts k)
-                            (($ $kargs (name) (var))
-                             (intmap-add functions kfun
-                                         (intset var (function-self kfun)))))
-                          functions))
-                     (($ $kargs _ _ ($ $continue k src
-                                       ($ $rec _ vars (($ $fun kfuns) ...))))
-                      (if (intset-ref single k)
-                          (fold (lambda (var kfun functions)
-                                  (intmap-add functions kfun
-                                              (intset var (function-self kfun))))
-                                functions vars kfuns)
-                          functions))
-                     (_ functions)))
-                 conts
-                 empty-intmap)))
+  (let* ((single (compute-singly-referenced-labels conts))
+         (first-class (compute-first-class-functions conts))
+         (first-class-defs (persistent-intset
+                            (intmap-fold (lambda (kfun def all-defs)
+                                           (intset-add! all-defs def))
+                                         first-class
+                                         empty-intset)))
+         (by-label (compute-functions-called-by-label conts)))
+    (define (first-class-bound-names)
+      (intset-fold
+       (lambda (kdef bound-names)
+         (match (intmap-ref conts kdef)
+           (($ $kargs _ _ ($ $continue k src ($ $fun kfun)))
+            (if (intset-ref single k)
+                (match (intmap-ref conts k)
+                  (($ $kargs (name) (var))
+                   (intmap-add bound-names kfun
+                               (intset var (function-self kfun)))))
+                bound-names))
+           (($ $kargs _ _ ($ $continue k src
+                             ($ $rec _ vars (($ $fun kfuns) ...))))
+            (if (intset-ref single k)
+                (fold (lambda (var kfun bound-names)
+                        (intmap-add bound-names kfun
+                                    (intset var (function-self kfun))))
+                      bound-names vars kfuns)
+                bound-names))))
+       first-class-defs
+       empty-intmap))
+    (define (add-second-class-functions bound-names)
+      (intset-fold
+       (lambda (label bound-names)
+         (cond
+          ((intmap-ref first-class label (lambda (_) #f))
+           ;; This function which is called by label also has
+           ;; first-class uses.  Either the bound names are known, in
+           ;; which case the label is in bound-names, or they aren't, in
+           ;; which case they aren't.  Either way the presence of $callk
+           ;; doesn't change the contifiability of a first-class
+           ;; function.
+           bound-names)
+          (else
+           ;; Otherwise this function is second-class: it has no value
+           ;; and is only called by label.  No bound names, but a
+           ;; candidate for contification nonetheless.
+           (intmap-add bound-names label empty-intset))))
+       by-label
+       bound-names))
+    (persistent-intmap
+     (add-second-class-functions
+      (first-class-bound-names)))))
 
 (define (compute-arities conts functions)
   "Given the map FUNCTIONS whose keys are $kfun labels, return a map
@@ -108,7 +146,11 @@ from label to arities."
     (if clause
         (match (intmap-ref conts clause)
           (($ $kclause arity body alt)
-           (cons arity (clause-arities alt))))
+           (cons arity (clause-arities alt)))
+          (($ $kargs names vars _)
+           ;; If this function's entry is a $kargs, all callers have
+           ;; compatible arity; no need to check.
+           #f))
         '()))
   (intmap-map (lambda (label vars)
                  (match (intmap-ref conts label)
@@ -137,12 +179,7 @@ from label to arities."
 functions with known uses that are only ever used as the operator of a
 $call, and are always called with a compatible arity."
   (let* ((functions (compute-functions conts))
-         (vars (intmap-fold (lambda (label vars out)
-                              (intset-fold (lambda (var out)
-                                             (intmap-add out var label))
-                                           vars out))
-                            functions
-                            empty-intmap))
+         (vars (invert-partition functions))
          (arities (compute-arities conts functions)))
     (define (restrict-arity functions proc nargs)
       (match (intmap-ref vars proc (lambda (_) #f))
@@ -188,11 +225,13 @@ $call, and are always called with a compatible arity."
               ;; compiler handles this fine though, so we allow it.
               (restrict-arity functions proc (length args))))
            (($ $callk k proc args)
-            (exclude-vars functions (cons proc args)))
+            (exclude-vars functions (if proc (cons proc args) args)))
            (($ $primcall name param args)
             (exclude-vars functions args))))
         (($ $kargs _ _ ($ $branch kf kt src op param args))
          (exclude-vars functions args))
+        (($ $kargs _ _ ($ $switch kf kt* src arg))
+         (exclude-var functions arg))
         (($ $kargs _ _ ($ $prompt k kh src escape? tag))
          (exclude-var functions tag))
         (($ $kargs _ _ ($ $throw src op param args))
@@ -231,6 +270,10 @@ function set."
              (let ((caller (intmap-ref bodies label (lambda (_) 0))))
                (values (intmap-add calls caller callee intset-add)
                        (intmap-add returns callee k intset-add))))))
+         (($ $kargs _ _ ($ $continue k src ($ $callk callee)))
+          (let ((caller (intmap-ref bodies label (lambda (_) 0))))
+            (values (intmap-add calls caller callee intset-add)
+                    (intmap-add returns callee k intset-add))))
          (_ (values calls returns))))
      conts
      (intset->intmap (lambda (label) empty-intset) (intset-add labels 0))
@@ -350,7 +393,8 @@ function set."
            empty-intset
            empty-intmap))
       (lambda (contified return-substs)
-        (values (intset-fold (lambda (label call-substs)
+        (values contified
+                (intset-fold (lambda (label call-substs)
                                (intset-fold
                                 (lambda (var call-substs)
                                   (intmap-add call-substs var label))
@@ -360,7 +404,7 @@ function set."
                              empty-intmap)
                 return-substs)))))
 
-(define (apply-contification conts call-substs return-substs)
+(define (apply-contification conts contified call-substs return-substs)
   (define (call-subst proc)
     (intmap-ref call-substs proc (lambda (_) #f)))
   (define (return-subst k)
@@ -430,7 +474,7 @@ function set."
           (match (intmap-ref conts k*)
             (($ $kreceive ($ $arity req () rest () #f) kargs)
              (match exp
-               (($ $call)
+               ((or ($ $call) ($ $callk))
                 (with-cps cps (build-term ($continue k* src ,exp))))
                ;; We need to punch through the $kreceive; otherwise we'd
                ;; have to rewrite as a call to the 'values primitive.
@@ -438,6 +482,26 @@ function set."
                 (inline-return cps k* kargs src (length req) rest vals))))
             (($ $ktail)
              (with-cps cps (build-term ($continue k* src ,exp))))))))
+  (define (contify-unchecked-function cps kfun)
+    ;; Precondition: kfun is "unchecked": the entry is a $kargs, and
+    ;; thus all callers are $callk.  If the front-end changes to produce
+    ;; $callk to a $kfun with $kclause, this will have to change.
+    (match (intmap-ref cps kfun)
+      (($ $kfun src meta self tail entry)
+       ;; This is the first caller to be visited; twiddle the kfun
+       ;; to be a $kargs with an additional closure arg if needed.
+       (match (intmap-ref cps entry)
+         (($ $kargs names vars term)
+          (let* ((vars' (map (lambda (_) (fresh-var)) vars))
+                 (names+ (if self (cons 'closure names) names))
+                 (vars+ (if self (cons self vars') vars')))
+            (with-cps cps
+              (setk kfun ($kargs names+ vars+
+                           ($continue entry src ($values vars')))))))))
+      (($ $kargs names vars)
+       ;; Callee $kfun already replaced with $kargs of the right
+       ;; arity.
+       cps)))
   (define (visit-exp cps k src exp)
     (match exp
       (($ $call proc args)
@@ -448,6 +512,15 @@ function set."
           (let ((body (find-body kfun (length args))))
             (with-cps cps
               (build-term ($continue body src ($values args))))))))
+      (($ $callk kfun proc args)
+       ;; If proc is contifiable, replace call with jump.
+       (cond
+        ((intset-ref contified kfun)
+         (let ((args (if proc (cons proc args) args)))
+           (with-cps (contify-unchecked-function cps kfun)
+             (build-term ($continue kfun src ($values args))))))
+        (else
+         (continue cps k src exp))))
       (($ $fun kfun)
        ;; If the function's tail continuation has been
        ;; substituted, that means it has been contified.
@@ -465,7 +538,7 @@ function set."
     (match term
       (($ $continue k src exp)
        (visit-exp cps k src exp))
-      ((or ($ $branch) ($ $prompt) ($ $throw))
+      ((or ($ $branch) ($ $switch) ($ $prompt) ($ $throw))
        (with-cps cps term))))
 
   ;; Renumbering is not strictly necessary but some passes may not be
@@ -494,5 +567,6 @@ function set."
   ;; conts as irreducible.  For now we punt and renumber so that there
   ;; are only live conts.
   (let ((conts (renumber conts)))
-    (let-values (((call-substs return-substs) (compute-contification conts)))
-      (apply-contification conts call-substs return-substs))))
+    (call-with-values (lambda () (compute-contification conts))
+      (lambda (contified call-substs return-substs)
+        (apply-contification conts contified call-substs return-substs)))))
